/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/
#include <project.h>
#include <stdio.h>
#include "adc.h"

 volatile uint32 dataReady = 0u;
  volatile int16 result[ADC_SAR_SEQ_TOTAL_CHANNELS_NUM];
  volatile uint32 timer_delay = 0u;

uint32 intr_status;
uint32 range_status;

CY_ISR(ISR_TIMER_LOC)
{
    timer_delay++;
    
    /* Measures temperature once per second */
    if(timer_delay > DELAY_1SEC)
    {
        /* Enables injection channel for next scan */
        ADC_SAR_SEQ_EnableInjection();
        timer_delay = 0u;
    }    
    
    /* Clears interrupt request from terminal count */
    PWM_ClearInterrupt(PWM_INTR_MASK_TC);
}


/******************************************************************************
* Function Name: ADC_SAR_SEQ_ISR_LOC
*******************************************************************************
*
* Summary:
*  Handle Interrupt Service Routine. Source - ADC SAR Seq.
*
******************************************************************************/
CY_ISR(ADC_SAR_SEQ_ISR_LOC)
{
    uint32 intr_status;
    uint32 range_status;

    /* Read interrupt status registers */
    intr_status = ADC_SAR_SEQ_SAR_INTR_MASKED_REG;
    /* Check for End of Scan interrupt */
    if((intr_status & ADC_SAR_SEQ_EOS_MASK) != 0u)
    {
        /* Read range detect status */
        range_status = ADC_SAR_SEQ_SAR_RANGE_INTR_MASKED_REG;
        /* Verify that the conversion result met the condition Low_Limit <= Result < High_Limit  */
        if((range_status & (uint32)(1ul << CH0_N)) != 0u) 
        {
            /* Read conversion result */
            result[CH0_N] = ADC_SAR_SEQ_GetResult16(CH0_N);
            /* Set PWM compare from channel0 */
            PWM_WriteCompare(result[CH0_N]);
        }    

        /* Clear range detect status */
        ADC_SAR_SEQ_SAR_RANGE_INTR_REG = range_status;
        dataReady |= ADC_SAR_SEQ_EOS_MASK;
    }    

    /* Check for Injection End of Conversion */
    if((intr_status & ADC_SAR_SEQ_INJ_EOC_MASK) != 0u)
    {
        result[TEMP_CH] = ADC_SAR_SEQ_GetResult16(TEMP_CH);
        dataReady |= ADC_SAR_SEQ_INJ_EOC_MASK;
    }    

    /* Clear handled interrupt */
    ADC_SAR_SEQ_SAR_INTR_REG = intr_status;
}


/* [] END OF FILE */
